import sys
import os
import getopt



if len(sys.argv) < 4:
	print("""
Usage) python find_longest_isoform.py [Fasta] [Gene - Isoforms] [Output fasta] (Options)

  - [Gene - Isoforms] file format
    geneA [tab] protein1 [tab] protein2 [tab] protein3 ...
    geneB [tab] protein4 [tab] protein5 [tab] protein6 ...

  - Options
    --keep-unnamed : Keep proteins not on the [Gene - Isoforms] file
    --delimiter [Character] : Set delimiter. 
                              i.e) --delimiter ' '  or --delimiter '|'
""")	
	sys.exit(2)

inputfasta = sys.argv[1]
xref = sys.argv[2]
outputfasta = sys.argv[3]
delimiter = " "
SKIP_UNNAMED = True

try:
	opts, args = getopt.getopt(sys.argv[4:], "", ["keep-unnamed","delimiter="])
except getopt.GetoptError:
	print("Error - Arguments")
	sys.exit(2)
for opt, arg in opts:
	if opt == '--keep-unnamed':
		SKIP_UNNAMED = False
	elif opt == '--delimiter':
		delimiter = arg



gene2isoforms = dict()
isoform2gene = dict() 
with open(xref,"r") as fp:
	for line in fp:
		linearray = line.rstrip().split("\t")
		gene = linearray.pop(0)
		gene2isoforms[gene] = linearray
		for isoform in linearray:
			isoform2gene[isoform] = gene


name, ext = os.path.splitext('inputfasta')
if ext == 'gz':
	import gzip
	try:
		fp = gzip.open(inputfasta,"rt")
	except:
		print("Error - Fail to open")
		sys.exit(2)
else:
	try:
		fp = open(inputfasta,"r")
	except:
		print("Error - Fail to open")
		sys.exit(2)


gene2length = {g:0 for g in gene2isoforms}
gene2seq = {g:"" for g in gene2isoforms}
gene2longest = {g:"" for g in gene2isoforms}
skip = False
currentseq = ""
currentlength = 0
for line in fp:
	line = line.rstrip()

	if line[0] == '>':
		line = line[1:]
		skip=False
		if currentlength != 0:
			if currentlength > gene2length[geneid]:
				gene2longest[geneid] = proteinid
				gene2length[geneid] = currentlength
				gene2seq[geneid] = currentseq
		
		currentlength = 0
		currentseq = ""
		proteinid = line.split(delimiter,1)[0]
		print(proteinid)
		if proteinid in isoform2gene:
			geneid = isoform2gene[proteinid]

		else:
			if SKIP_UNNAMED == True:
				skip=True
			else:
				geneid = proteinid
				gene2isoforms[proteinid] = [proteinid]
				gene2seq[proteinid] = ""
				gene2length[proteinid] = 0
	else:
		if skip==True:
			continue
		currentseq += line+"\n"
		currentlength += len(line)

with open(outputfasta,"w") as fout:
	for geneid in gene2longest:
		fout.write(">%s\n"%gene2longest[geneid])
		fout.write(gene2seq[geneid])
			

fp.close()
